import { __extends, __read, __spread } from "tslib";
import { EOrientation } from './internal/interfaces';
import ViewLayoutContainer, { HTMLView, NodeView } from './internal/ViewLayoutContainer';
import SplitLayoutContainer from './internal/SplitLayoutContainer';
import LineUpLayoutContainer from './internal/LineUpLayoutContainer';
import TabbingLayoutContainer from './internal/TabbingLayoutContainer';
import RootLayoutContainer from './internal/RootLayoutContainer';
var ABuilder = /** @class */ (function () {
    function ABuilder() {
        this._name = 'View';
        this._fixed = false;
        this._autoWrap = false;
        this._fixedLayout = false;
    }
    /**
     * specify the name of the view
     * @param {string} name the new name
     * @return {this} itself
     */
    ABuilder.prototype.name = function (name) {
        this._name = name;
        return this;
    };
    /**
     * specify that the view cannot be closed and the view and separators cannot be moved via drag and drop
     * setting the fixed option implies the fixedLayout option
     * @return {this} itself
     */
    ABuilder.prototype.fixed = function () {
        this._fixed = true;
        this._fixedLayout = true;
        return this;
    };
    /**
     * specify that drag and drop is disabled for views, but the separator can still be moved
     * @returns {this}
     */
    ABuilder.prototype.fixedLayout = function () {
        this._fixedLayout = true;
        return this;
    };
    /**
     * specify that the view should be automatically wrapped with a tabbing container in case of a new split
     * @return {this} itself
     */
    ABuilder.prototype.autoWrap = function (name) {
        this._autoWrap = name !== undefined ? name : true;
        return this;
    };
    ABuilder.prototype.buildOptions = function () {
        return {
            name: this._name,
            fixed: this._fixed,
            autoWrap: this._autoWrap,
            fixedLayout: this._fixedLayout
        };
    };
    return ABuilder;
}());
var ViewBuilder = /** @class */ (function (_super) {
    __extends(ViewBuilder, _super);
    function ViewBuilder(view) {
        var _this = _super.call(this) || this;
        _this.view = view;
        _this._hideHeader = false;
        return _this;
    }
    ViewBuilder.prototype.hideHeader = function () {
        this._hideHeader = true;
        this._fixed = true;
        return this;
    };
    ViewBuilder.prototype.buildOptions = function () {
        return Object.assign({
            hideHeader: this._hideHeader
        }, _super.prototype.buildOptions.call(this));
    };
    ViewBuilder.prototype.build = function (root, doc) {
        var options = this.buildOptions();
        if (typeof this.view === 'string') {
            return new ViewLayoutContainer(new HTMLView(this.view, doc), options);
        }
        if (this.view.nodeName !== undefined) {
            return new ViewLayoutContainer(new NodeView(this.view), options);
        }
        return new ViewLayoutContainer(this.view, options);
    };
    return ViewBuilder;
}(ABuilder));
export { ViewBuilder };
function toBuilder(view) {
    if (view instanceof ABuilder) {
        return view;
    }
    return new ViewBuilder(view);
}
var AParentBuilder = /** @class */ (function (_super) {
    __extends(AParentBuilder, _super);
    function AParentBuilder(children) {
        var _this = _super.call(this) || this;
        _this.children = [];
        _this._name = 'Container';
        children.forEach(function (c) { return _this.push(c); });
        return _this;
    }
    AParentBuilder.prototype.push = function (view) {
        this.children.push(toBuilder(view));
        return this;
    };
    AParentBuilder.prototype.buildChildren = function (root, doc) {
        return this.children.map(function (c) { return c.build(root, doc); });
    };
    return AParentBuilder;
}(ABuilder));
var SplitBuilder = /** @class */ (function (_super) {
    __extends(SplitBuilder, _super);
    function SplitBuilder(orientation, ratio, left, right) {
        var _this = _super.call(this, [left, right]) || this;
        _this.orientation = orientation;
        _this._ratio = 0.5;
        _this._ratio = ratio;
        return _this;
    }
    /**
     * set the ratio between the left and right view
     * @param {number} ratio the new ratio
     * @return {SplitBuilder} itself
     */
    SplitBuilder.prototype.ratio = function (ratio) {
        this._ratio = ratio;
        return this;
    };
    SplitBuilder.prototype.buildOptions = function () {
        return Object.assign({
            orientation: this.orientation,
        }, _super.prototype.buildOptions.call(this));
    };
    SplitBuilder.prototype.build = function (root, doc) {
        if (doc === void 0) { doc = document; }
        var built = this.buildChildren(root, doc);
        console.assert(built.length >= 2);
        var r = new SplitLayoutContainer(doc, this.buildOptions(), this._ratio, built[0], built[1]);
        built.slice(2).forEach(function (c) { return r.push(c); });
        return r;
    };
    return SplitBuilder;
}(AParentBuilder));
var LineUpBuilder = /** @class */ (function (_super) {
    __extends(LineUpBuilder, _super);
    function LineUpBuilder(orientation, children, stackLayout) {
        if (stackLayout === void 0) { stackLayout = false; }
        var _this = _super.call(this, children) || this;
        _this.orientation = orientation;
        _this.stackLayout = stackLayout;
        return _this;
    }
    /**
     * push another child
     * @param {IBuildAbleOrViewLike} view the view to add
     * @return {LineUpBuilder} itself
     */
    LineUpBuilder.prototype.push = function (view) {
        return _super.prototype.push.call(this, view);
    };
    LineUpBuilder.prototype.buildOptions = function () {
        return Object.assign({
            orientation: this.orientation,
            stackLayout: this.stackLayout
        }, _super.prototype.buildOptions.call(this));
    };
    LineUpBuilder.prototype.build = function (root, doc) {
        if (doc === void 0) { doc = document; }
        var built = this.buildChildren(root, doc);
        return new (LineUpLayoutContainer.bind.apply(LineUpLayoutContainer, __spread([void 0, doc, this.buildOptions()], built)))();
    };
    return LineUpBuilder;
}(AParentBuilder));
var TabbingBuilder = /** @class */ (function (_super) {
    __extends(TabbingBuilder, _super);
    function TabbingBuilder() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._active = null;
        return _this;
    }
    /**
     * push another tab
     * @param {IBuildAbleOrViewLike} view the tab
     * @return {TabbingBuilder} itself
     */
    TabbingBuilder.prototype.push = function (view) {
        return _super.prototype.push.call(this, view);
    };
    /**
     * adds another child and specify it should be the active one
     * @param {IBuildAbleOrViewLike} view the active tab
     * @return {AParentBuilder} itself
     */
    TabbingBuilder.prototype.active = function (view) {
        this._active = this.children.length;
        return _super.prototype.push.call(this, view);
    };
    TabbingBuilder.prototype.buildOptions = function () {
        return Object.assign({
            active: this._active
        }, _super.prototype.buildOptions.call(this));
    };
    TabbingBuilder.prototype.build = function (root, doc) {
        var built = this.buildChildren(root, doc);
        return new (TabbingLayoutContainer.bind.apply(TabbingLayoutContainer, __spread([void 0, doc, this.buildOptions()], built)))();
    };
    return TabbingBuilder;
}(AParentBuilder));
/**
 * builder for creating a horizontal split layout (moveable splitter)
 * @param {number} ratio ratio between the two given elements
 * @param {IBuildAbleOrViewLike} left left container
 * @param {IBuildAbleOrViewLike} right right container
 * @return {SplitBuilder} a split builder
 */
export function horizontalSplit(ratio, left, right) {
    return new SplitBuilder(EOrientation.HORIZONTAL, ratio, left, right);
}
/**
 * builder for creating a vertical split layout (moveable splitter)
 * @param {number} ratio ratio between the two given elements
 * @param {IBuildAbleOrViewLike} left left container
 * @param {IBuildAbleOrViewLike} right right container
 * @return {SplitBuilder} a split builder
 */
export function verticalSplit(ratio, left, right) {
    return new SplitBuilder(EOrientation.VERTICAL, ratio, left, right);
}
/**
 * builder for creating a horizontal lineup layout (each container has the same full size with scrollbars)
 * @param {IBuildAbleOrViewLike} children the children of the layout
 * @return {LineUpBuilder} a lineup builder
 */
export function horizontalLineUp() {
    var children = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        children[_i] = arguments[_i];
    }
    return new LineUpBuilder(EOrientation.HORIZONTAL, children);
}
/**
 * builder for creating a vertical lineup layout (each container has the same full size with scrollbars)
 * @param {IBuildAbleOrViewLike} children the children of the layout
 * @return {LineUpBuilder} a lineup builder
 */
export function verticalLineUp() {
    var children = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        children[_i] = arguments[_i];
    }
    return new LineUpBuilder(EOrientation.VERTICAL, children);
}
/**
 * similar to the horizontalLineUp, except that each container takes its own amount of space
 * @param {IBuildAbleOrViewLike} children the children of the layout
 * @return {LineUpBuilder} a lineup builder
 */
export function horizontalStackedLineUp() {
    var children = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        children[_i] = arguments[_i];
    }
    return new LineUpBuilder(EOrientation.HORIZONTAL, children, true);
}
/**
 * similar to the verticalLineUp, except that each container takes its own amount of space
 * @param {IBuildAbleOrViewLike} children the children of the layout
 * @return {LineUpBuilder} a lineup builder
 */
export function verticalStackedLineUp() {
    var children = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        children[_i] = arguments[_i];
    }
    return new LineUpBuilder(EOrientation.VERTICAL, children, true);
}
/**
 * builder for creating a tab layout
 * @param {IBuildAbleOrViewLike} children the children of the layout
 * @return {TabbingBuilder} a tabbing builder
 */
export function tabbing() {
    var children = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        children[_i] = arguments[_i];
    }
    return new TabbingBuilder(children);
}
/**
 * builder for creating a view
 * @param {string | IView} view possible view content
 * @return {ViewBuilder} a view builder
 */
export function view(view) {
    return new ViewBuilder(view);
}
/**
 * creates the root of a new layout
 * @param {IBuildAbleOrViewLike} child the only child of the root
 * @param {Document} doc root Document
 * @return {IRootLayoutContainer} the root element
 */
export function root(child, doc) {
    if (doc === void 0) { doc = document; }
    var b = toBuilder(child);
    var r = new RootLayoutContainer(doc, function (child) { return toBuilder(child).build(r, doc); }, function (dump, restoreView) { return restore(dump, restoreView, doc); });
    r.root = b.build(r, doc);
    return r;
}
/**
 * restores the given layout dump
 * @param {ILayoutDump} dump the dump
 * @param {(referenceId: number) => IView} restoreView lookup function for getting the underlying view given the dumped reference id
 * @param {Document} doc root document
 * @return {ILayoutContainer} the root element
 */
export function restore(dump, restoreView, doc) {
    if (doc === void 0) { doc = document; }
    var restorer = function (d) { return restore(d, restoreView, doc); };
    switch (dump.type) {
        case 'root':
            return RootLayoutContainer.restore(dump, doc, function (r, child) { return toBuilder(child).build(r, doc); }, function (dump, restoreView) { return restore(dump, restoreView, doc); }, restoreView);
        case 'split':
            return SplitLayoutContainer.restore(dump, restorer, doc);
        case 'lineup':
            return LineUpLayoutContainer.restore(dump, restorer, doc);
        case 'tabbing':
            return TabbingLayoutContainer.restore(dump, restorer, doc);
        case 'view':
            return ViewLayoutContainer.restore(dump, restoreView, doc);
        default:
            throw new Error("invalid type: " + dump.type);
    }
}
/**
 * derives from an existing html scaffolded layout the phovea layout and replaced the nodes with it
 * @param {HTMLElement} node the root node
 * @param {(node: HTMLElement) => IView} viewFactory how to build a view from a node
 */
export function derive(node, viewFactory) {
    if (viewFactory === void 0) { viewFactory = function (node) { return new NodeView(node); }; }
    var doc = node.ownerDocument;
    var r = new RootLayoutContainer(doc, function (child) { return toBuilder(child).build(r, doc); }, function (dump, restoreView) { return restore(dump, restoreView, doc); });
    var deriveImpl = function (node) {
        switch (node.dataset.layout || 'view') {
            case 'hsplit':
            case 'vsplit':
            case 'split':
                return SplitLayoutContainer.derive(node, deriveImpl);
            case 'lineup':
            case 'vlineup':
            case 'hlineup':
            case 'stack':
            case 'hstack':
            case 'vstack':
                return LineUpLayoutContainer.derive(node, deriveImpl);
            case 'tabbing':
                return TabbingLayoutContainer.derive(node, deriveImpl);
            default:
                // interpret as view
                return ViewLayoutContainer.derive(viewFactory(node) || new NodeView(node));
        }
    };
    r.root = deriveImpl(node);
    if (node.parentElement) {
        //replace old node with new root
        node.parentElement.replaceChild(r.node, node);
    }
    return r;
}
//# sourceMappingURL=builder.js.map